@@ -1,22 +1,23 @@
 import warnings
 
 import numpy as np
+import pandas
 from napari import Viewer
 from napari_tools_menu import register_function
 import napari
 import math
 from ._all_frames import analyze_all_frames
 
-def regionprops(image_layer : napari.layers.Layer, labels_layer: napari.layers.Labels, napari_viewer : Viewer, size : bool = True, intensity : bool = True, perimeter : bool = False, shape : bool = False, position : bool = False, moments : bool = False):
+def regionprops(image_layer : napari.layers.Layer, labels_layer: napari.layers.Labels, size : bool = True, intensity : bool = True, perimeter : bool = False, shape : bool = False, position : bool = False, moments : bool = False, napari_viewer : Viewer = None):
     warnings.warn("napari_skimage_regionprops.regionprops is deprecated. Use regionprops_table instead.")
     image_data = None
     if image_layer is not None:
         image_data = image_layer.data
 
-    regionprops_table(image_data, labels_layer.data, napari_viewer, size, intensity, perimeter, shape, position, moments)
+    regionprops_table(image_data, labels_layer.data, size, intensity, perimeter, shape, position, moments, napari_viewer)
 
-@register_function(menu="Measurement > Regionprops (nsr)")
-def regionprops_table(image : napari.types.ImageData, labels: napari.types.LabelsData, napari_viewer : Viewer = None, size : bool = True, intensity : bool = True, perimeter : bool = False, shape : bool = False, position : bool = False, moments : bool = False):
+@register_function(menu="Measurement > Regionprops (scikit-image, nsr)")
+def regionprops_table(image : napari.types.ImageData, labels: napari.types.LabelsData, size : bool = True, intensity : bool = True, perimeter : bool = False, shape : bool = False, position : bool = False, moments : bool = False, napari_viewer : Viewer = None) -> "pandas.DataFrame":
     """
     Adds a table widget to a given napari viewer with quantitative analysis results derived from an image-label pair.
     """
@@ -71,6 +72,11 @@ def regionprops_table(image : napari.types.ImageData, labels: napari.types.Label
         properties = properties + ['solidity', 'extent', 'feret_diameter_max', 'local_centroid']
         if len(labels.shape) == 2:
             properties = properties + ['major_axis_length', 'minor_axis_length', 'orientation', 'eccentricity']
+            # we need these two to compute some shape descriptors
+            if not size:
+                properties = properties + ['area']
+            if not perimeter:
+                properties = properties + ['perimeter']
         else:
             properties = properties + ['moments_central']
         # euler_number,
@@ -98,7 +104,14 @@ def regionprops_table(image : napari.types.ImageData, labels: napari.types.Label
                               properties=properties, extra_properties=extra_properties)
 
     if shape:
-        if len(labels.data.shape) == 3:
+        if len(labels.shape) == 2:
+            # See https://imagej.nih.gov/ij/docs/menus/analyze.html
+            table['aspect_ratio'] = table['major_axis_length'] / table['minor_axis_length']
+            table['roundness'] = 4 * table['area'] / np.pi / pow(table['major_axis_length'], 2)
+            table['circularity'] = 4 * np.pi * table['area'] / pow(table['perimeter'], 2)
+
+        # 3D image
+        if len(labels.shape) == 3:
             axis_lengths_0 = []
             axis_lengths_1 = []
             axis_lengths_2 = []
@@ -125,6 +138,11 @@ def regionprops_table(image : napari.types.ImageData, labels: napari.types.Label
                 # remove moment from table as we didn't ask for them
                 table = {k: v for k, v in table.items() if not 'moments_central' in k}
 
+        if not size:
+            table = {k: v for k, v in table.items() if k != 'area'}
+        if not perimeter:
+            table = {k: v for k, v in table.items() if k != 'perimeter'}
+
     if napari_viewer is not None:
         # Store results in the properties dictionary:
         labels_layer.properties = table
@@ -133,7 +151,8 @@ def regionprops_table(image : napari.types.ImageData, labels: napari.types.Label
         from ._table import add_table
         add_table(labels_layer, napari_viewer)
     else:
-        return table
+        import pandas
+        return pandas.DataFrame(table)
 
 def ellipsoid_axis_lengths(table):
     """Compute ellipsoid major, intermediate and minor axis length.
@@ -165,3 +184,39 @@ def ellipsoid_axis_lengths(table):
 
 regionprops_table_all_frames = analyze_all_frames(regionprops_table)
 register_function(regionprops_table_all_frames, menu="Measurement > Regionprops of all frames (nsr)")
+
+
+
+try:
+    # morphometrics API
+    from morphometrics.measure import register_measurement_set
+
+    # this is just a wrapper with the right image parameter names
+    def regionprops_for_morphometrics(intensity_image : "napari.types.ImageData", label_image: "napari.types.LabelsData", size : bool = True, intensity : bool = True, perimeter : bool = False, shape : bool = False, position : bool = False, moments : bool = False) -> "pd.DataFrame":
+        return regionprops_table(intensity_image, label_image,
+                                 size=size,
+                                 intensity=intensity,
+                                 shape=shape,
+                                 perimeter=perimeter,
+                                 position=position,
+                                 moments=moments)
+
+    register_measurement_set(
+        regionprops_for_morphometrics,
+        name="regionprops (nsr)",
+        choices=["intensity", "size", "shape", "perimeter", "position", "moments"],
+        uses_intensity_image=True,
+    )
+
+    from napari_tools_menu import register_dock_widget
+    from morphometrics._gui._qt.measurement_widgets import QtMeasurementWidget
+    register_dock_widget(
+        QtMeasurementWidget,
+        "Measurement > Region properties (morphometrics)"
+    )
+except:
+    pass
+
+
+
+
